@@ -1,11 +1,12 @@
-# pylint: disable=missing-docstring,invalid-name,too-few-public-methods,misplaced-comparison-constant
+# pylint: disable=missing-docstring,invalid-name,too-few-public-methods
 
 import datetime
+import json
 import os
 import tempfile
 from contextlib import contextmanager
 from pathlib import Path
-from typing import List, Optional, Union
+from typing import Iterable, List, Optional, Union
 from unittest.mock import MagicMock, patch
 
 import click
@@ -15,7 +16,7 @@ from sqlalchemy import create_engine, func
 from sqlalchemy.orm import sessionmaker
 
 import libratom
-import libratom.cli.subcommands as subcommands
+from libratom.cli import subcommands
 from libratom.cli.cli import ratom
 from libratom.cli.utils import (
     install_spacy_model,
@@ -24,12 +25,20 @@ from libratom.cli.utils import (
     validate_existing_dir,
     validate_version_string,
 )
+from libratom.lib.base import AttachmentMetadata
 from libratom.lib.constants import SPACY_MODELS
 from libratom.lib.core import load_spacy_model, open_mail_archive
 from libratom.lib.database import db_session_from_cmd_out
 from libratom.lib.entities import process_message
 from libratom.lib.utils import BodyType, cleanup_message_body
-from libratom.models import Configuration, Entity, FileReport, Message
+from libratom.models import (
+    Configuration,
+    Entity,
+    FileReport,
+    HeaderField,
+    HeaderFieldType,
+    Message,
+)
 
 
 @contextmanager
@@ -42,7 +51,7 @@ class Expected:
     Result object type for parametrized tests. Expand as necessary...
     """
 
-    def __init__(self, status: int, tokens: List[str], **kwargs):
+    def __init__(self, status: int, tokens: Iterable[str] = (), **kwargs):
         self.status = status
         self.tokens = tokens
 
@@ -61,7 +70,7 @@ def run_ratom_subcommand(
     Block of code to run a given ratom subcommand as part of a test
     """
 
-    if not str:
+    if not cmd:
         raise ValueError("Empty ratom subcommand string")
 
     subcommand = [cmd]
@@ -277,8 +286,14 @@ def test_ratom_report_empty(isolated_cli_runner, params, expected):
                 tokens=["spaCy model", "installed version", "available versions"],
             ),
         ),
-        (["-i"], Expected(status=2, tokens=["Error: -i option requires an argument"])),
-        (["-u"], Expected(status=2, tokens=["Error: -u option requires an argument"])),
+        (
+            ["-i"],
+            Expected(status=2, tokens=["Error: Option '-i' requires an argument."]),
+        ),
+        (
+            ["-u"],
+            Expected(status=2, tokens=["Error: Option '-u' requires an argument."]),
+        ),
         (
             ["-li", "en_core_web_sm"],
             Expected(
@@ -295,8 +310,8 @@ def test_ratom_model(cli_runner, params, expected):
 @pytest.mark.parametrize(
     "params, expected",
     [
-        (["-i", "en_core_web_sm"], Expected(status=0, tokens=[])),
-        (["-u", "en_core_web_sm"], Expected(status=0, tokens=[])),
+        (["-i", "en_core_web_sm"], Expected(status=0)),
+        (["-u", "en_core_web_sm"], Expected(status=0)),
     ],
 )
 def test_ratom_model_install(cli_runner, params, expected):
@@ -319,9 +334,8 @@ def test_ratom_entities_from_mbox_files(
 
 
 @pytest.mark.skipif(
-    not os.getenv("CONTINUOUS_INTEGRATION", None)
-    or os.getenv("TRAVIS_OS_NAME", None) == "osx",
-    reason="Keep local test runs and OSX travis runs reasonably short",
+    not os.getenv("CI", None),
+    reason="Keep local test runs reasonably short",
 )
 @pytest.mark.parametrize(
     "params, expected",
@@ -335,7 +349,7 @@ def test_ratom_entities_from_mbox_files(
 def test_ratom_entities_enron_004(
     isolated_cli_runner,
     enron_dataset_part004,
-    en_core_web_sm_3_0_0,  # pylint: disable=unused-argument
+    en_core_web_sm_3_4_1,  # pylint: disable=unused-argument
     params,
     expected,
 ):
@@ -350,7 +364,7 @@ def test_ratom_entities_enron_004(
             assert str(entity)
 
         # Verify total entity count
-        assert session.query(Entity).count() == 188276
+        assert session.query(Entity).count() == 216758
 
         # Verify count per entity type
         results = (
@@ -362,24 +376,24 @@ def test_ratom_entities_enron_004(
         assert results
 
         expected_counts = {
-            "CARDINAL": 29954,
-            "DATE": 8078,
-            "EVENT": 54,
-            "FAC": 234,
-            "GPE": 6539,
-            "LANGUAGE": 1,
-            "LAW": 191,
-            "LOC": 306,
-            "MONEY": 1609,
-            "NORP": 431,
-            "ORDINAL": 520,
-            "ORG": 120699,
-            "PERCENT": 827,
-            "PERSON": 14532,
-            "PRODUCT": 660,
-            "QUANTITY": 95,
-            "TIME": 2976,
-            "WORK_OF_ART": 570,
+            "CARDINAL": 43484,
+            "DATE": 8798,
+            "EVENT": 69,
+            "FAC": 360,
+            "GPE": 6952,
+            "LANGUAGE": 3,
+            "LAW": 361,
+            "LOC": 287,
+            "MONEY": 1914,
+            "NORP": 763,
+            "ORDINAL": 613,
+            "ORG": 122184,
+            "PERCENT": 13135,
+            "PERSON": 13235,
+            "PRODUCT": 633,
+            "QUANTITY": 253,
+            "TIME": 3095,
+            "WORK_OF_ART": 619,
         }
 
         for entity_type, count in results:
@@ -391,14 +405,106 @@ def test_ratom_entities_enron_004(
             .filter_by(name="spacy_model_version")
             .one()
             .value
-            == "3.0.0"
+            == "3.4.1"
         )
 
 
+@pytest.mark.parametrize(
+    "command",
+    ["entities", "report"],
+)
+@pytest.mark.parametrize(
+    "params, expected_counts",
+    [
+        (
+            [],
+            {HeaderFieldType: 0, HeaderField: 0},
+        ),
+        (
+            ["-m"],
+            {HeaderFieldType: 145, HeaderField: 16144},
+        ),
+    ],
+)
+def test_ratom_commands_with_header_fields(
+    isolated_cli_runner,
+    enron_dataset_part001,
+    en_core_web_sm_3_4_1,  # pylint: disable=unused-argument
+    command,
+    params,
+    expected_counts,
+):
+    # All runs should be successful
+    success = Expected(status=0)
+
+    # Add verbose flag to get the DB file from cmd out
+    result = run_ratom_subcommand(
+        command, params + ["-v"], enron_dataset_part001, isolated_cli_runner, success
+    )
+
+    with db_session_from_cmd_out(result) as session:
+
+        # Validate row counts for header field types and header fields
+        for object_type, count in expected_counts.items():
+            assert session.query(object_type).count() == count
+
+
+@pytest.mark.parametrize(
+    "params, expected",
+    [
+        (
+            ["-v", "-j2"],
+            Expected(status=0, tokens=["Creating database file", "All done"]),
+        )
+    ],
+)
+def test_ratom_entities_eml_files(
+    isolated_cli_runner,
+    test_eml_files,
+    en_core_web_sm_3_4_1,  # pylint: disable=unused-argument
+    params,
+    expected,
+):
+    # Bad date string in example13.eml but valid for entity extraction
+    valid_eml_files = test_eml_files / "emails" / "rfc2822"
+
+    result = extract_entities(params, valid_eml_files, isolated_cli_runner, expected)
+
+    with db_session_from_cmd_out(result) as session:
+
+        # Verify total entity count
+        assert session.query(Entity).count() == 86
+
+        # Verify count per entity type
+        results = (
+            session.query(Entity.label_, func.count(Entity.label_))
+            .group_by(Entity.label_)
+            .all()
+        )
+
+        assert results
+
+        expected_counts = {
+            "CARDINAL": 16,
+            "DATE": 20,
+            "FAC": 1,
+            "GPE": 2,
+            "LAW": 3,
+            "MONEY": 1,
+            "ORG": 4,
+            "PRODUCT": 2,
+            "PERSON": 29,
+            "TIME": 13,
+            "WORK_OF_ART": 5,
+        }
+
+        for entity_type, count in results:
+            assert expected_counts[entity_type] == count
+
+
 @pytest.mark.skipif(
-    not os.getenv("CONTINUOUS_INTEGRATION", None)
-    or os.getenv("TRAVIS_OS_NAME", None) == "osx",
-    reason="Keep local test runs and OSX travis runs reasonably short",
+    not os.getenv("CI", None),
+    reason="Keep local test runs reasonably short",
 )
 @pytest.mark.parametrize(
     "params, expected",
@@ -502,7 +608,7 @@ def test_process_message():
             "date": datetime.datetime.utcnow(),
             "body": "hello",
             "body_type": BodyType.PLAIN,
-            "spacy_model": None,
+            "spacy_model_name": None,
             "attachments": None,
         }
     )
@@ -551,17 +657,142 @@ def test_validate_eml_export_input(bad_eml_export_input):
         validate_eml_export_input(None, None, bad_eml_export_input)
 
 
-def test_ratom_emldump(cli_runner, enron_dataset_part004, good_eml_export_input):
-    expected = Expected(status=0, tokens=[])
-
+def test_ratom_emldump_from_pff(
+    cli_runner, enron_dataset_part004, good_eml_export_input
+):
     with tempfile.TemporaryDirectory() as tmpdir:
+        root_path = Path(tmpdir)
         params = [
             f"-l{enron_dataset_part004}",
             f"-o{tmpdir}",
             str(good_eml_export_input),
         ]
 
-        dump_eml_files(params, None, cli_runner, expected)
+        # Run ratom emldump command
+        dump_eml_files(params, None, cli_runner, Expected(status=0))
+
+        # Confirm exported data
+        expected = {
+            "andy_zipper_000_1_1": {
+                "2203588": [
+                    AttachmentMetadata(
+                        name="US Gas Stack & Website for New Version 121401.xls",
+                        size=39936,
+                    ),
+                ],
+                "2203620": [
+                    AttachmentMetadata(name="AGA.xls", size=129024),
+                ],
+            },
+            "andy_zipper_001_1_1": {
+                "2174116": [
+                    AttachmentMetadata(name="Positions_10_15.xls", size=23040),
+                    AttachmentMetadata(name="Positions_10_16.xls", size=23040),
+                    AttachmentMetadata(name="Positions_10_17.xls", size=23040),
+                    AttachmentMetadata(name="Positions_10_18.xls", size=23040),
+                    AttachmentMetadata(name="Positions_10_19.xls", size=23040),
+                    AttachmentMetadata(name="Positions_10_22.xls", size=23552),
+                    AttachmentMetadata(name="Positions_10_23.xls", size=23552),
+                ]
+            },
+        }
+
+        for file_dir, msg_dirs in expected.items():
+            for msg_pff_identifier, attachments in msg_dirs.items():
+
+                # Confirm eml file is there
+                eml_file_path = root_path / file_dir / f"{msg_pff_identifier}.eml"
+                assert eml_file_path.is_file()
+
+                for attachment in attachments:
+
+                    # Confirm attachment files are there
+                    attachment_path = (
+                        root_path
+                        / file_dir
+                        / f"{msg_pff_identifier}_attachments"
+                        / attachment.name
+                    )
+                    assert attachment_path.is_file()
+                    assert attachment_path.stat().st_size == attachment.size
+
+
+def test_ratom_emldump_from_mbox(cli_runner, directory_of_mbox_files):
+
+    ratom_emldump_input = [
+        {
+            "filename": "201901.mbox",
+            "sha256": "70a405404fd766a...",
+            "id_list": ["6", "7", "12", "19", "57", "62", "68", "85", "85", "104"],
+        }
+    ]
+
+    with tempfile.TemporaryDirectory() as tmpdir:
+        root_path = Path(tmpdir)
+
+        # Make json input file
+        json_file_path = root_path / "ratom_emldump_input.json"
+        with json_file_path.open(mode="w", encoding="UTF-8") as json_file:
+            json.dump(ratom_emldump_input, json_file)
+
+        params = [
+            f"-l{directory_of_mbox_files}",
+            f"-o{tmpdir}",
+            str(json_file_path),
+        ]
+
+        # Run ratom emldump command
+        dump_eml_files(params, None, cli_runner, Expected(status=0))
+
+        # Confirm exported data
+        expected = {
+            "201901": {
+                "6": [
+                    AttachmentMetadata(name="image001.png", size=5598),
+                ],
+                "7": [
+                    AttachmentMetadata(name="image001.png", size=5598),
+                ],
+                "12": [
+                    AttachmentMetadata(name="image001.png", size=5598),
+                ],
+                "19": [
+                    AttachmentMetadata(name="image001.png", size=5598),
+                ],
+                "57": [
+                    AttachmentMetadata(name="image001.png", size=5598),
+                ],
+                "62": [
+                    AttachmentMetadata(name="httpd.conf", size=21138),
+                ],
+                "68": [
+                    AttachmentMetadata(name="httpd.conf", size=21219),
+                ],
+                "85": [
+                    AttachmentMetadata(name="meet.galex-713.eu.conf", size=2382),
+                    AttachmentMetadata(name="prosody.cfg.lua", size=8705),
+                ],
+                "104": [
+                    AttachmentMetadata(name="error.log_err", size=143406),
+                ],
+            },
+        }
+
+        for file_dir, msg_dirs in expected.items():
+            for msg_id, attachments in msg_dirs.items():
+
+                # Confirm eml file is there
+                eml_file_path = root_path / file_dir / f"{msg_id}.eml"
+                assert eml_file_path.is_file()
+
+                for attachment in attachments:
+
+                    # Confirm attachment files are there
+                    attachment_path = (
+                        root_path / file_dir / f"{msg_id}_attachments" / attachment.name
+                    )
+                    assert attachment_path.is_file()
+                    assert attachment_path.stat().st_size == attachment.size
 
 
 def test_install_spacy_model():
