@@ -3,19 +3,22 @@ import datetime
 import email
 import hashlib
 import logging
-from email import policy
+import os
+import sys
+import textwrap
+from email import message_from_string, policy
 from pathlib import Path
 from tempfile import TemporaryDirectory
-from unittest.mock import MagicMock, Mock, patch
+from unittest.mock import DEFAULT, MagicMock, Mock, patch
 
 import pytest
+from github import Github
 
 import libratom
 from libratom.data import MIME_TYPES
 from libratom.lib.concurrency import get_messages
 from libratom.lib.constants import SPACY_MODELS, BodyType
 from libratom.lib.core import (
-    extract_message_from_archive,
     get_cached_spacy_model,
     get_set_of_files,
     open_mail_archive,
@@ -111,10 +114,19 @@ def test_get_transport_headers_from_sent_items(enron_dataset_part004):
 def test_extract_message_attachments(enron_dataset_part002):
     """Checking 3 known attachments, to validate the attachment extraction process"""
 
-    digests = {
-        47685: "d48232614b01e56014293854abbb5db3",
-        47717: "cf8be7cd3e6e14307972246e2942c9d1",
-        47749: "081e6b66dc89671ff6460adac94dbab1",
+    attachments = {
+        47685: {
+            "name": "cubicle hurdles.mpeg",
+            "digest": "d48232614b01e56014293854abbb5db3",
+        },
+        47717: {
+            "name": "Hallway races.mpeg",
+            "digest": "cf8be7cd3e6e14307972246e2942c9d1",
+        },
+        47749: {
+            "name": "Rowing.mpeg",
+            "digest": "081e6b66dc89671ff6460adac94dbab1",
+        },
     }
 
     with PffArchive(
@@ -135,8 +147,13 @@ def test_extract_message_attachments(enron_dataset_part002):
             )
             filepath.write_bytes(rbuf)
 
-            # Confirm checksum
-            assert hashlib.md5(rbuf).hexdigest() == digests[att.identifier]
+            # Confirm attachment name
+            assert att.name == attachments[att.identifier]["name"]
+
+            # Confirm attachment checksum
+            assert (
+                hashlib.md5(rbuf).hexdigest() == attachments[att.identifier]["digest"]
+            )
 
             # Sanity check on the file
             assert filepath.stat().st_size == att.size
@@ -157,6 +174,34 @@ def test_get_messages_with_bad_files(enron_dataset_part044, mock_progress_callba
     assert _count == 558
 
 
+def test_get_messages_with_bad_message(sample_pst_file, mock_progress_callback):
+
+    _count = 0
+
+    with patch.object(
+        libratom.lib.pff.PffArchive, "get_attachment_metadata"
+    ) as patched_method:
+
+        # Raise on the first call, then don't raise
+        def side_effects():
+            yield RuntimeError
+            while True:
+                yield DEFAULT
+
+        patched_method.side_effect = side_effects()
+
+        for _count, res in enumerate(
+            get_messages(
+                files=[sample_pst_file],
+                progress_callback=mock_progress_callback,
+            ),
+            start=1,
+        ):
+            assert res
+
+    assert _count == 2667
+
+
 def test_get_message_by_id(sample_pst_file):
     with PffArchive(sample_pst_file) as archive:
         for message in archive.messages():
@@ -178,11 +223,25 @@ def test_file_report_with_empty_relationship():
     assert file_report.processing_wall_time is None
 
 
-@pytest.mark.parametrize(
-    "model_name, expected_entity_types",
-    [("en_core_web_trf", {"DATE", "GPE", "ORG", "PERSON", "QUANTITY"})],
+@pytest.mark.skipif(
+    (sys.version_info.major, sys.version_info.minor, sys.version_info.micro)
+    >= (3, 10, 0)
+    and Github(os.environ.get("GITHUB_TOKEN"))
+    .get_repo("pytorch/pytorch")
+    .get_issue(number=66424)
+    .state
+    == "open",
+    reason="https://github.com/pytorch/pytorch/issues/66424",
 )
-def test_apply_spacy_model(sample_pst_file, model_name, expected_entity_types):
+@pytest.mark.parametrize(
+    "expected_entity_types",
+    [{"DATE", "ORG", "PERSON", "QUANTITY"}],
+)
+def test_apply_transformer_model(
+    sample_pst_file, en_core_web_trf_3_4_1, expected_entity_types
+):
+    model_name = en_core_web_trf_3_4_1.name
+
     # Extract a known (short) message
     msg_id = 2112164
     with open_mail_archive(sample_pst_file) as archive:
@@ -194,23 +253,20 @@ def test_apply_spacy_model(sample_pst_file, model_name, expected_entity_types):
     # Pre-load our model to install any missing dependencies
     assert get_cached_spacy_model(model_name)
 
-    # Apply our model pretending to be in a forked process
-    with patch("libratom.lib.entities.current_process") as mock_current_process:
-        mock_current_process.return_value.name = "NotMainProcess"
-
-        # pylint:disable=no-value-for-parameter
-        res, error = process_message(
-            # Must use dictionary form if function is called explicitly
-            {
-                "filepath": sample_pst_file,
-                "message_id": msg_id,
-                "date": datetime.datetime.utcnow(),
-                "body": msg_body,
-                "body_type": BodyType.PLAIN,
-                "spacy_model_name": model_name,
-                "attachments": None,
-            }
-        )
+    # Run worker function
+    # pylint:disable=no-value-for-parameter
+    res, error = process_message(
+        # Must use dictionary form if function is called explicitly
+        {
+            "filepath": sample_pst_file,
+            "message_id": msg_id,
+            "date": datetime.datetime.utcnow(),
+            "body": msg_body,
+            "body_type": BodyType.PLAIN,
+            "spacy_model_name": model_name,
+            "attachments": None,
+        }
+    )
 
     assert res and not error
 
@@ -327,8 +383,30 @@ def test_download_files_with_bad_urls():
             download_files(bad_urls, Path(tmpdir))
 
 
-def test_utf8_message_with_no_cte_header_as_string(utf8_message_with_no_cte_header):
-    assert MboxArchive.format_message(utf8_message_with_no_cte_header)
+def test_utf8_message_with_no_cte_header_as_string():
+    # Modified from https://github.com/python/cpython/blob/v3.10.8/Lib/test/test_email/test_email.py#L338
+    # Confirm that the text is properly encoded and that an "8bit" CTE is added.
+    msg = textwrap.dedent(
+        """\
+        MIME-Version: 1.0
+        Test if non-ascii messages with no Content-Type nor
+        Content-Transfer-Encoding set can be as_string'd:
+        Föö bär
+        """
+    )
+
+    expected = textwrap.dedent(
+        """\
+        MIME-Version: 1.0
+        content-transfer-encoding: 8bit
+
+        Test if non-ascii messages with no Content-Type nor
+        Content-Transfer-Encoding set can be as_string'd:
+        Föö bär
+        """
+    )
+
+    assert MboxArchive.format_message(message_from_string(msg)) == expected
 
 
 def test_get_file_info(sample_pst_file):
@@ -375,7 +453,7 @@ def test_get_mbox_message_by_id(sample_mbox_file):
 
         for index, message in enumerate(archive.messages(), start=1):
             msg = archive.get_message_by_id(index)
-            assert extract_message_from_archive(archive, index)
+            assert msg
             assert archive.format_message(msg) == archive.format_message(message)
             assert archive.get_message_headers(message)
 
