@@ -1,6 +1,7 @@
 # pylint: disable=missing-docstring
 import filecmp
 import os
+from importlib import resources
 from pathlib import Path
 from tempfile import TemporaryDirectory
 
@@ -9,12 +10,6 @@ import pytest
 from libratom import data
 from libratom.scripts.get_media_type_list import download_media_type_files
 
-try:
-    from importlib import resources
-except ImportError:
-    # backport version for Python 3.6
-    import importlib_resources as resources
-
 
 @pytest.mark.parametrize("params, expected", [(["-h"], "Usage"), (["--help"], "Usage")])
 def test_get_media_type_list_cli(cli_runner, params, expected):
@@ -39,5 +34,5 @@ def test_validate_media_type_list(cli_runner):
 
         # Prevent a IANA media types update from failing a CI build
         # Run the test for coverage but skip the assertion
-        if os.getenv("CONTINUOUS_INTEGRATION", "").lower() != "true":
+        if os.getenv("CI", "").lower() != "true":
             assert filecmp.cmp(existing_media_types_file, new_media_types_file)
